@@ -40,9 +40,8 @@ This module provides different methods to access and filter the statistics compi
 
 =head2 new()
 
-Create a new C<Compilation> object. This creator is only useful if you
-don't call C<get()> of C<Sys::Statistics::Linux>. You can create a new
-object with:
+Create a new C<Sys::Statistics::Linux::Compilation> object. This creator is only useful if you
+don't call C<get()> of C<Sys::Statistics::Linux>. You can create a new object with:
 
     my $lxs  = Sys::Statistics::Linux::LoadAVG->new();
     my $load = $lxs->get;
@@ -66,6 +65,8 @@ object with:
 
 =item netinfo()
 
+C<netinfo()> provides raw data - no deltas.
+
 =item sockstats()
 
 =item diskstats()
@@ -87,19 +88,14 @@ returns the first level keys of the statistics. Example:
     my @dev  = $stat->netstats;                 # the devices eth0, eth1, ...
     my $eth0 = $stat->netstats('eth0');         # eth0 statistics as a hash reference
     my @keys = $stat->netstats('eth0');         # the statistic keys
-    my @vals = $stat->netstats('eth0', @keys);  # the values for @keys
+    my @vals = $stat->netstats('eth0', @keys);  # the values for the passed device and @keys
+    my $val  = $stat->netstats('eth0', $key);   # the value for the passed device and key
 
-I was thinking about to return all keys sorted but if you need that you can call
+Sorted ...
 
     my @dev  = sort $stat->netstats;
     my @keys = sort $stat->netstats('eth0');
 
-LoadAVG example:
-
-    my $load = $stat->loadavg;                  # loadavg as a hash reference
-    my @keys = $stat->loadavg;                  # the statistic keys
-    my @vals = $stat->loadavg(@keys);           # the values for @keys
-
 =head2 pstop()
 
 This method is looking for top processes and returns a sorted list of PIDs as an array or
@@ -151,41 +147,11 @@ This would return the following matches:
     * all cpu where "iowait" is grather than 10
     * only disk '/dev/sda1' if "usageper" is grather than 80
 
-If the statistics are not gathered by the current process then you can handoff statistics as an
-argument.
-
-    my %stats = (
-        cpustats => {
-            cpu => {
-                system => '51.00',
-                total  => '51.00',
-                idle   => '49.00',
-                nice   => '0.00',
-                user   => '0.00',
-                iowait => '0.00'
-            }
-        }
-    );
-
-    my $stat = Sys::Statistics::Linux::Compilation(\%stats);
-
-    my %filter = (
-        cpustats => {
-            total => 'gt:50'
-        }
-    );
-
-    my $hits = $stat->search(\%filter);
-
 The method C<psfind()> scans for processes only and returns a array reference with all process
 IDs that matched the filter. Example:
 
     my $pids = $stat->psfind({ cmd => qr/init/, owner => 'eq:apache' });
 
-You can handoff the statistics as second argument as well.
-
-    my $pids = $stat->psfind(\%filter, \%stats);
-
 This would return the following process ids:
 
     * processes that matched the command "init"
@@ -227,7 +193,7 @@ Please report all bugs to <jschulz.cpan(at)bloonix.de>.
 
 Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
-Thanks to Moritz Lenz for his suggestion for the name for this module.
+Thanks to Moritz Lenz for his suggestion for the name of this module.
 
 =head1 COPYRIGHT
 
@@ -238,12 +204,13 @@ This program is free software; you can redistribute it and/or modify it under th
 =cut
 
 package Sys::Statistics::Linux::Compilation;
-our $VERSION = '0.02_01';
 
 use strict;
 use warnings;
 use Carp qw(croak);
 
+our $VERSION = '0.10';
+
 # Creating the statistics accessors
 BEGIN {
     foreach my $stat (qw/sysinfo procstats memstats sockstats loadavg filestats/) {
@@ -281,7 +248,7 @@ BEGIN {
 sub new {
     my ($class, $stats) = @_;
     unless (ref($stats) eq 'HASH') {
-        croak 'Usage: class->new( \%statistics )';
+        croak 'Usage: $class->new( \%statistics )';
     }
     return bless $stats, $class;
 }
@@ -309,24 +276,7 @@ sub search {
         # matched the searched string
 
         foreach my $x (keys %{$fref}) {
-
-            # if $fref->{$x} is a hash ref then the next key have to
-            # match the statistic key. this is used for statistics
-            # like NetStats or Processes that uses a hash key for the
-            # device name or process id. NetStats example... if
-            #
-            #    $fref->{eth0}->{ttbyt}
-            #
-            # is defined as a filter then the key "eth0" have to match
-            #
-            #    $proc->{eth0}
-            #
-            # then we look if "ttbyt" matched the searched statistic name
-            # and compare the values. if the comparing returns TRUE we the
-            # hash is copied until the matched key-value pair.
-
             if (ref($fref->{$x}) eq 'HASH') {
-
                 # if the key $proc->{eth0} doesn't exists
                 # then we continue with the next defined filter
                 next unless exists $proc->{$x};
@@ -341,7 +291,13 @@ sub search {
                 foreach my $key (keys %{$proc}) {
                     if (ref($proc->{$key}) eq 'HASH') {
                         $subref = $proc->{$key};
-                        if (defined $subref->{$x} && $self->_compare($subref->{$x}, $fref->{$x})) {
+                        if (ref $subref->{$x} eq 'HASH') {
+                            foreach my $y (keys %{$subref->{$x}}) {
+                                if ($self->_compare($subref->{$x}->{$y}, $fref->{$x})) {
+                                    $hits{$opt}{$key}{$x}{$y} = $subref->{$x}->{$y};
+                                }
+                            }
+                        } elsif (defined $subref->{$x} && $self->_compare($subref->{$x}, $fref->{$x})) {
                             $hits{$opt}{$key}{$x} = $subref->{$x};
                         }
                     } else { # must be a scalar now
@@ -354,6 +310,7 @@ sub search {
             }
         }
     }
+
     return wantarray ? %hits : \%hits;
 }
 
@@ -363,11 +320,21 @@ sub psfind {
     my $proc   = $self->{processes} or return undef;
     my @hits   = ();
 
-    foreach my $pid (keys %{$proc}) {
+    PID: foreach my $pid (keys %{$proc}) {
         my $proc = $proc->{$pid};
         while ( my ($key, $value) = each %{$filter} ) {
-            if (exists $proc->{$key} && $self->_compare($proc->{$key}, $value)) {
-                push @hits, $pid;
+            if (exists $proc->{$key}) {
+                if (ref $proc->{$key} eq 'HASH') {
+                    foreach my $v (values %{$proc->{$key}}) {
+                        if ($self->_compare($v, $value)) {
+                            push @hits, $pid;
+                            next PID;
+                        }
+                    }
+                } elsif ($self->_compare($proc->{$key}, $value)) {
+                    push @hits, $pid;
+                    next PID;
+                }
             }
         }
     }
@@ -397,7 +364,8 @@ sub pstop {
 #
 
 sub _compare {
-    my ($class, $x, $y) = @_;
+    my ($self, $x, $y) = @_;
+
     if (ref($y) eq 'Regexp') {
         return $x =~ $y;
     } elsif ($y =~ s/^eq://) {
@@ -409,8 +377,9 @@ sub _compare {
     } elsif ($y =~ s/^lt://) {
         return $x < $y;
     } else {
-        croak "$class: bad search() / psfind() operator '$y'";
+        croak ref($self).": bad search() / psfind() operator '$y'";
     }
+
     return undef;
 }
 
